/* common.h - Common functions/includes
 * Copyright 2000-2001 srvx Development Team
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: common.h,v 1.60.2.10 2001/09/10 19:11:49 entrope Exp $
 */

#ifndef _COMMON_H_
#define _COMMON_H_

#include "config.h"

#if defined(__GNUC__) && !defined(HAVE_ALLOCA_H)
# define alloca __builtin_alloca
#else
# if defined(HAVE_ALLOCA_H) && defined(CAN_INCLUDE_ALLOCA_H)
#  include <alloca.h>
# else
#  ifdef _AIX
#   pragma alloca
#  else
#   ifndef alloca
char *alloca();
#   endif
#  endif
# endif
#endif

#ifdef HAVE_SYS_TYPES_H
#include <sys/types.h>
#endif

#define SOCKCHECK_USABLE (defined(HAVE_POLL) && defined(HAVE_PTHREAD) && !defined(__CYGWIN__))

#include "proto.h"
#include "messages.h"
#include "compat.h"

#if !defined(HAVE_LOCALTIME_R) && !defined(__CYGWIN__)
extern struct tm *localtime_r(const time_t *clock, struct tm *res);
#elif defined(__CYGWIN__)
# define localtime_r(clock, res) memcpy(res, localtime(clock), sizeof(struct tm));
#endif

#ifndef true
#define true 1
#endif

#ifndef false
#define false 0
#endif

#ifndef INADDR_NONE
#define INADDR_NONE 0xffffffffL
#endif

#define ArrayLength(x)		(sizeof(x)/sizeof(x[0]))
#define safestrncpy(dest, src, len) do { char *d = (dest); unsigned int l = (len); strncpy(d, (src), l); d[l-1] = 0; } while (0)

#if WITH_DMALLOC
#define DMALLOC_FUNC_CHECK 1
#include <string.h>
#include <dmalloc.h>
#endif

extern time_t now;

struct userNode *AddService(const char *nick, const char *desc);
struct userNode *AddClone(const char *nick, const char *username, const char *hostname, const char *desc);

int create_socket_client(struct uplinkNode *target);
int read_from_socket(void);
void close_socket(void);

typedef void (*exit_func_t)(void);
void reg_exit_func(exit_func_t handler);
void call_exit_funcs(void);

typedef void (*db_write_func_t)(void);
void reg_db_write_func(db_write_func_t handler);
void write_databases(void);

typedef void (*oper_func_t) (struct userNode *user);
void reg_oper_func(oper_func_t handler);

const char* inttobase64(char* buf, unsigned int v, unsigned int count);
unsigned long base64toint(const char* s,const int count);
int getusermodes(char *sender);
void mod_usermode(struct userNode *user, const char *modes);
void mod_chanmode(struct chanNode *channel, const char *mode_change, char *key, int limit);
int verify_chanmode(const char *modes);
int split_line(unsigned char *line, int irc_colon, int argv_size, unsigned char *argv[]);

#define match_ircglobs !mmatch

int mmatch(const char *glob, const char *newglob);
int match_ircglob(const char *text, const char *glob);
int user_matches_glob(struct userNode *user, const char *glob, int include_nick);

int is_ircmask(const char *text);
int is_gline(const char *text);

/* The "default" for generate_hostmask is to have all of these options off. */
#define GENMASK_STRICT   1
#define GENMASK_USENICK  2
#define GENMASK_OMITNICK 4  /* Hurray for Kevin! */
#define GENMASK_BYIP     8
#define GENMASK_SRVXMASK 16
char *generate_hostmask(struct userNode *user, int options);
unsigned long ParseInterval(const unsigned char *interval);
unsigned long ParseVolume(const unsigned char *volume);
int parse_ipmask(const unsigned char *str, unsigned long *addr, unsigned long *mask);
#define MATCH_IPMASK(test, addr, mask) (((test & mask) ^ (addr & mask)) == 0)

#define MD5_CRYPT_LENGTH 33
/* buffer[] must be at least MD5_CRYPT_LENGTH bytes long */
const char *cryptpass(const char *pass, char buffer[]);
int checkpass(const char *pass, const char *crypt);

int split_ircmask(char *text, char **nick, char **ident, char **host);
void unsplit_string(unsigned char *set[], unsigned int max);

#define DECLARE_LIST(STRUCTNAME,ITEMTYPE) struct STRUCTNAME {\
  unsigned int used, size;\
  ITEMTYPE* list;\
};\
void STRUCTNAME##_init(struct STRUCTNAME *list);\
void STRUCTNAME##_append(struct STRUCTNAME *list, ITEMTYPE new_item);\
int STRUCTNAME##_remove(struct STRUCTNAME *list, ITEMTYPE new_item);\
void STRUCTNAME##_clean(struct STRUCTNAME *list)

#define DEFINE_LIST(STRUCTNAME,ITEMTYPE) \
void STRUCTNAME##_init(struct STRUCTNAME *list) {\
  list->used = 0;\
  list->size = 8;\
  list->list = malloc(list->size*sizeof(list->list[0]));\
}\
void STRUCTNAME##_append(struct STRUCTNAME *list, ITEMTYPE new_item) {\
  if (list->used == list->size) {\
    list->size <<= 1;\
    list->list = realloc(list->list, list->size*sizeof(list->list[0]));\
  }\
  list->list[list->used++] = new_item;\
}\
int STRUCTNAME##_remove(struct STRUCTNAME *list, ITEMTYPE new_item) {\
    unsigned int n, found;\
    for (found=n=0; n<list->used; n++) {\
	if (list->list[n] == new_item) {\
	    memmove(list->list+n, list->list+n+1, (list->used-n-1)*sizeof(list->list[n]));\
	    found = 1;\
	    list->used--;\
	}\
    }\
    return found;\
}\
void STRUCTNAME##_clean(struct STRUCTNAME *list) {\
  list->used = list->size = 0;\
  free(list->list);\
}

/* The longest string that's likely to be produced is "10 minutes, and 10
   seconds." (27 characters) */
#define INTERVALLEN	32

void intervalString(char *output, time_t interval);
int getipbyname(const char *name, unsigned long *ip);

const char * preposition(char *word);
int set_policer_param(const char *param, void *data, void *extra);

DECLARE_LIST(string_buffer, char);
void string_buffer_append_string(struct string_buffer *buf, const char *tail);
void string_buffer_replace(struct string_buffer *buf, unsigned int from, unsigned int len, const char *repl);

#define enabled_string(string)	(!strcasecmp((string), "on") || !strcmp((string), "1") || !strcasecmp((string), "enabled"))
#define disabled_string(string)	(!strcasecmp((string), "off") || !strcmp((string), "0") || !strcasecmp((string), "disabled"))

#endif /* ifdef _COMMON_H_ */
